Unit _JTPE;
// code JPEG

(*
 TPE 2001-2002
 DODANE Jean-Baptiste, ROUSSILLON Cyril  TS3
 Lyce Xavier Marmier, PONTARLIER

 Pour toute question, vous pouvez nous contacter :
   Jean-Baptiste (LZW, RLE) : jb@jbdsoftware.fr.st
   Cyril (JPEG, prdictif)  : cyril@crtek.fr.st
   
 Les TPE complets sont disponibles  l'adresse :
   www.chez.com/itek
 Ou sur nos sites personnels :
   www.jbdsoftware.fr.st
   www.crteknologies.fr.st
*)

Interface

Uses
  Windows, Graphics, SysUtils, Dialogs, Classes, _Bibliotheque;

Type
  sYCbCr = Class
  Public
    Y, Cb, Cr: Byte;
    Procedure FromRGB(RGB: TColor);
    Function ToRGB: TColor;
  End;

	Procedure ChargerJTPE(NomFichier: String; Var Img: sImage; Progress: TProgressEvent; Fichier: pFichier);
	Procedure SauverJTPE(NomFichier: String; Var Img: sImage; Qualite: Byte; Progress: TProgressEvent);
	Procedure DCT(Var Source, Resultat: Tab8x8);
	Procedure IDCT(Var Source, Resultat: Tab8x8);

Implementation

Const
	RLE = 128; Gd = 129; zero = 64;
  ZZ: Array[0..63, 0..1] Of Byte = ((0,0),(1,0),(0,1),(0,2),(1,1),(2,0),(3,0),(2,1),
  																	(1,2),(0,3),(0,4),(1,3),(2,2),(3,1),(4,0),(5,0),
                                    (4,1),(3,2),(2,3),(1,4),(0,5),(0,6),(1,5),(2,4),
                                    (3,3),(4,2),(5,1),(6,0),(7,0),(6,1),(5,2),(4,3),
                                    (3,4),(2,5),(1,6),(0,7),(1,7),(2,6),(3,5),(4,4),
                                    (5,3),(6,2),(7,1),(7,2),(6,3),(5,4),(4,5),(3,6),
                                    (2,7),(3,7),(4,6),(5,5),(6,4),(7,3),(7,4),(6,5),
                                    (5,6),(4,7),(5,7),(6,6),(7,5),(7,6),(6,7),(7,7));

  TQuantD: Array[0..1, 0..7, 0..7] Of Single = (((16,11,10,16,24,40,51,61),
  																							 (12,12,14,19,26,58,60,55),
                                                 (14,13,16,24,40,57,69,56),
                                                 (14,17,22,29,51,87,80,62),
                                                 (18,22,37,56,68,109,103,77),
                                                 (24,35,55,64,81,104,113,92),
                                                 (49,64,78,87,103,121,120,101),
                                                 (72,92,95,98,112,100,103,99)),
                                                ((17,18,24,47,99,99,99,99),
                                                 (18,21,26,66,99,99,99,99),
                                                 (24,26,56,99,99,99,99,99),
                                                 (47,66,99,99,99,99,99,99),
                                                 (99,99,99,99,99,99,99,99),
                                                 (99,99,99,99,99,99,99,99),
                                                 (99,99,99,99,99,99,99,99),
                                                 (99,99,99,99,99,99,99,99)));

Var
	Arbre: sHuffman;
  TQuant: Array[0..1, 0..7, 0..7] Of Single;
  DC: Array[0..2] Of Integer;
  TempR, TempC: Byte;


//---------------------------------------------------------------------------------------------
// Luminance / chrominance

// sYCbCr.FromRGB
Procedure sYCbCr.FromRGB(RGB: TColor);
Var Yt, Cbt, Crt: SmallInt;
Begin
 	Yt := Round(0.299 * GetRValue(RGB) + 0.587 * GetGValue(RGB) + 0.114 * GetBValue(RGB));
	Cbt := Round(-0.1687 * GetRValue(RGB) - 0.3313 * GetGValue(RGB) + 0.5 * GetBValue(RGB)) + 128;
	Crt := Round(0.5 * GetRValue(RGB) - 0.4187 * GetGValue(RGB) - 0.0813 * GetBValue(RGB)) + 128;
  If Yt > 255 Then Y := 255 Else If Yt < 0 Then Y := 0 Else Y := Yt;
  If Cbt > 255 Then Cb := 255 Else If Cbt < 0 Then Cb := 0 Else Cb := Cbt;
  If Crt > 255 Then Cr := 255 Else If Crt < 0 Then Cr := 0 Else Cr := Crt;
End;

// sYIQ.ToRGB
Function sYCbCr.ToRGB: TColor;
Var R, G, B: SmallInt;
Begin
	R := Round(Y + 1.402 * (Cr - 128));
	G := Round(Y - 0.34414 * (Cb - 128) - 0.71414 * (Cr - 128));
	B := Round(Y + 1.772 * (Cb - 128));
  If R > 255 Then R := 255 Else If R < 0 Then R := 0;
  If G > 255 Then G := 255 Else If G < 0 Then G := 0;
  If B > 255 Then B := 255 Else If B < 0 Then B := 0;
  Result := RGB(R, G, B);
End;

//*********************************************************************************************
// Compression
//*********************************************************************************************


//---------------------------------------------------------------------------------------------
// DCT - Transforme en cosinus de 'Source' dans 'Resultat'
Procedure DCT(Var Source, Resultat: Tab8x8);
Const
	UnSurRac2 = 0.7071067812;
Var
	i, j, x, y: Integer;
  Coeff, Fast1, Fast2, Temp: Double;
Begin
	For i := 0 To 7 Do For j := 0 To 7 Do
  Begin
    Coeff := 1;
    If i = 0 Then Coeff := Coeff * UnSurRac2;
    If j = 0 Then Coeff := Coeff * UnSurRac2;
    Fast1 := i * Pi / 8;
    Fast2 := j * Pi / 8;
    Temp := 0;
  	For x := 0 To 7 Do For y := 0 To 7 Do
			Temp := Temp + Source[x, y] * cos((x + 0.5) * Fast1) * cos((y + 0.5) * Fast2);
    Resultat[i, j] := Round(Coeff * 0.25 * Temp);
  End
End;


//---------------------------------------------------------------------------------------------
// EcrireBloc - Ecrit le bloc de 8x8 pixels 'Tableau' dans fichier 'HandleF', 'Quant'=Y/Cb/Cr
Function EcrireBloc(Tableau: Tab8x8; nbL, nbC, Quant, HandleF: Integer): Integer;
Var
	i, j, k: Integer;
{}  x, y, nbZero, taille, Quant2: Integer;
  TabR: Tab8x8;
  TabLin: Array[0..127] Of Byte;
Begin
	If (nbC = 0) Or (nbL = 0) Then Begin Result := 0; Exit End;
	Case nbC Of
    7: Begin Result := EcrireBloc(Tableau, nbL, 4, Quant, HandleF);
       For j := 0 To 7 Do For i := 0 To 2 Do Tableau[i, j] := Tableau[i + 4, j];
       Result := Result + EcrireBloc(Tableau, nbL, 3, Quant, HandleF); Exit End;
    6: Begin Result := EcrireBloc(Tableau, nbL, 4, Quant, HandleF);
       For j := 0 To 7 Do For i := 0 To 1 Do Tableau[i, j] := Tableau[i + 4, j];
       Result := Result + EcrireBloc(Tableau, nbL, 2, Quant, HandleF); Exit End;
    5: Begin Result := EcrireBloc(Tableau, nbL, 4, Quant, HandleF);
       For j := 0 To 7 Do Tableau[0, j] := Tableau[4, j];
       Result := Result + EcrireBloc(Tableau, nbL, 1, Quant, HandleF); Exit End;
    4: For j := 0 To 7 Do For i := 0 To 3 Do Tableau[7 - i, j] := Tableau[i, j];
    3: Begin Result := EcrireBloc(Tableau, nbL, 2, Quant, HandleF);
       For j := 0 To 7 Do Tableau[0, j] := Tableau[2, j];
       Result := Result + EcrireBloc(Tableau, nbL, 1, Quant, HandleF); Exit End;
    2: For j := 0 To 7 Do
       Begin Tableau[2, j] := Tableau[1, j]; Tableau[3, j] := Tableau[0, j];
             Tableau[4, j] := Tableau[0, j]; Tableau[5, j] := Tableau[1, j];
             Tableau[6, j] := Tableau[1, j]; Tableau[7, j] := Tableau[0, j]; End;
    1: For j := 0 To 7 Do For i := 1 To 7 Do Tableau[i, j] := Tableau[0, j];
  End;
	Case nbL Of
    7: Begin Result := EcrireBloc(Tableau, 4, nbC, Quant, HandleF);
       For j := 0 To 7 Do For i := 0 To 2 Do Tableau[j, i] := Tableau[j, i + 4];
       Result := Result + EcrireBloc(Tableau, 3, nbC, Quant, HandleF); Exit End;
    6: Begin Result := EcrireBloc(Tableau, 4, nbC, Quant, HandleF);
       For j := 0 To 7 Do For i := 0 To 1 Do Tableau[j, i] := Tableau[j, i + 4];
       Result := Result + EcrireBloc(Tableau, 2, nbC, Quant, HandleF); Exit End;
    5: Begin Result := EcrireBloc(Tableau, 4, nbC, Quant, HandleF);
       For j := 0 To 7 Do Tableau[j, 0] := Tableau[j, 4];
       Result := Result + EcrireBloc(Tableau, 1, nbC, Quant, HandleF); Exit End;
    4: For j := 0 To 7 Do For i := 0 To 3 Do Tableau[j, 7 - i] := Tableau[j, i];
    3: Begin Result := EcrireBloc(Tableau, 2, nbC, Quant, HandleF);
       For j := 0 To 7 Do Tableau[j, 0] := Tableau[j, 2];
       Result := Result + EcrireBloc(Tableau, 1, nbC, Quant, HandleF); Exit End;
    2: For j := 0 To 7 Do
       Begin Tableau[j, 2] := Tableau[j, 1]; Tableau[j, 3] := Tableau[j, 0];
             Tableau[j, 4] := Tableau[j, 0]; Tableau[j, 5] := Tableau[j, 1];
             Tableau[j, 6] := Tableau[j, 1]; Tableau[j, 7] := Tableau[j, 0]; End;
    1: For j := 0 To 7 Do For i := 1 To 7 Do Tableau[j, i] := Tableau[j, 0];
  End;

  DCT(Tableau, TabR);
  //--- Quantifier + DPCM
  If Quant = 0 Then Quant2 := 0 Else Quant2 := 1;
  For i := 0 To 7 Do For j := 0 To 7 Do
    TabR[i, j] := Round(TabR[i, j] / TQuant[Quant2][i, j]) + zero;
  i := TabR[0, 0] - zero; Dec(TabR[0, 0], DC[Quant]); DC[Quant] := i; // DPCM
  //--- Linariser zigzag + RLE (<=> 0) + Huffman adaptative
  Arbre.Actualiser; nbZero := 0;
	taille := TempR; TabLin[0] := TempC;

  For i := 0 To 63 Do
  BEGIN
    x := ZZ[i, 0]; y := ZZ[i, 1];
    // Ne pas crire l'lment s'il n'est pas utile (pixels bords)
    If nbL = 4 Then If (x Mod 2) <> 0 Then Continue Else Else
    If nbL = 2 Then If (x Mod 4) <> 0 Then Continue Else Else
    If nbL = 1 Then If x <> 0 Then Continue;
    If nbC = 4 Then If (y Mod 2) <> 0 Then Continue Else Else
    If nbC = 2 Then If (y Mod 4) <> 0 Then Continue Else Else
    If nbC = 1 Then If y <> 0 Then Continue;
    //--- RLE
    If TabR[x, y] = zero Then Inc(nbZero) Else
    Begin
	    //--- Fin RLE
	    If nbZero <> 0 Then
	    Begin
	      If nbZero > 2 Then
	      Begin
	        Arbre.EcrireCar(RLE, taille, TabLin);
	        EcrireBits(nbZero - 1, 6, taille, TabLin);
	      End Else For j := 1 To nbZero Do Arbre.EcrireCar(zero, taille, TabLin);
	      nbZero := 0;
	    End;
	    //--- Dpassement tendue
	   	If (TabR[x, y] < 0) Or (TabR[x, y] > (zero * 2) - 1) Then
	    Begin
	    	Arbre.EcrireCar(Gd, taille, TabLin);
	    	Inc(TabR[x, y], 2048 - zero);
	      EcrireBits(TabR[x, y], 12, taille, TabLin);
	    End Else Arbre.EcrireCar(TabR[x, y], taille, TabLin);
    End;
  END;

  If nbZero <> 0 Then If nbZero > 2 Then
    Begin
      Arbre.EcrireCar(RLE, taille, TabLin);
      EcrireBits(nbZero - 1, 6, taille, TabLin);
    End Else For j := 1 To nbZero Do Arbre.EcrireCar(zero, taille, TabLin);

  Result := taille Div 8;
	TempR := taille Mod 8; TempC := TabLin[Result];
  FileWrite(HandleF, TabLin, Result);
End;


//---------------------------------------------------------------------------------------------
// EcrireMBloc - Ecrit un bloc de 16x16 de 'Image' donn par 'R' dans fichier 'HandleF'
Function EcrireMBloc(Var Img: sImage; R: TRect; HandleF: Integer): Integer;
Var
	i, j, k, l, Count: Integer;
  TempCb, TempCr: Single;
  Ta1, Ta2, Ta3: Tab8x8;
  Pixel: TColor;
  W, H, C, x, y: Integer;
Begin
	Count := 0;
	//--- Chrominance
  W := (R.Right - R.Left); H := (R.Bottom - R.Top);
	For j := 0 To H Div 2 Do For i := 0 To W Div 2 Do
  Begin
  	TempCb := 0; TempCr := 0; C := 0;
    For l := 0 To 1 Do For k := 0 To 1 Do
    Begin
    	If (R.Left + i * 2 + l > R.Right) Or (R.Top + j * 2 + k > R.Bottom) Then Begin Inc(C); Continue End;
			Pixel := Img.Pix[R.Left + i * 2 + l, R.Top + j * 2 + k];
     	TempCb := TempCb + (-0.1687 * GetRValue(Pixel) - 0.3313 * GetGValue(Pixel) + 0.5 * GetBValue(Pixel));
			TempCr := TempCr + (0.5 * GetRValue(Pixel) - 0.4187 * GetGValue(Pixel) - 0.0813 * GetBValue(Pixel));
    End;
    Ta2[i, j] := Round(TempCb / (4 - C)) + 128;
    Ta3[i, j] := Round(TempCr / (4 - C)) + 128;
  End;
  Inc(Count, EcrireBloc(Ta2, H Div 2 + 1, W Div 2 + 1, 1, HandleF));
  Inc(Count, EcrireBloc(Ta3, H Div 2 + 1, W Div 2 + 1, 2, HandleF));
	//--- Luminance
	For l := 0 To H Div 8 Do For k := 0 To W Div 8 Do
  Begin
  	For j := 0 To 7 Do For i := 0 To 7 Do
    Begin
    	If (R.Left + k * 8 + i > R.Right) Or (R.Top + l * 8 + j > R.Bottom) Then Break;
			Pixel := Img.Pix[R.Left + k * 8 + i, R.Top + l * 8 + j];
      Ta1[i, j] := Round(0.299 * GetRValue(Pixel) + 0.587 * GetGValue(Pixel) + 0.114 * GetBValue(Pixel));
    End;
    y := ((H Mod 8 + 1) * (1 - H Div 8) + 8 * (H Div 8)) * (1 - k) + (H Mod 8 + 1) * (H Div 8) * k;
    x := ((W Mod 8 + 1) * (1 - W Div 8) + 8 * (W Div 8)) * (1 - l) + (W Mod 8 + 1) * (W Div 8) * l;
    Inc(Count, EcrireBloc(Ta1, y, x, 0, HandleF));
  End;
  Result := Count;
End;


//---------------------------------------------------------------------------------------------
// SauverJTPE - Enregistre 'Image' dans fichier 'NomFichier' de qualit 'Qualite'
Procedure SauverJTPE(NomFichier: String; Var Img: sImage; Qualite: Byte; Progress: TProgressEvent);
Var
	i, j, m, n, x, y: Integer;
  Adresse: Integer;
  HandleF: Integer;
	Qual: Single;
begin
	Arbre := sHuffman.Create(130);
  Adresse := 17;
  DC[0] := 0; DC[1] := 0; DC[2] := 0;
  //--- Cration tables de quantification
  If Qualite > 100 Then Qualite := 100;  Qual := 2000 / (Qualite + 16.21) - 17.21022287;
  If Qualite = 0 Then For i := 0 To 7 Do For j := 0 To 7 Do
  	Begin TQuant[0][i, j] := TQuantD[0][i, j]; TQuant[1][i, j] := TQuantD[1][i, j] End Else
  For i := 0 To 7 Do For j := 0 To 7 Do
  Begin
  	TQuant[0][i, j] := 1 + Qual * (1 + i + j);
  	TQuant[1][i, j] := 1 + Qual * (1 + Sqr(i) + Sqr(j));
  End;
  //--- Ecriture en-tte fichier
  HandleF := FileCreate(NomFichier);
  FileWrite(HandleF, 'JTPE', 4);
  FileWrite(HandleF, Adresse, 4);
  FileWrite(HandleF, Img.Largeur, 4);
  FileWrite(HandleF, Img.Hauteur, 4);
  FileWrite(HandleF, Qualite, 1);
  //---
  TempR := 0;
	For n := 0 To (Img.Hauteur - 1) Div 16 Do
  BEGIN
  	// Ligne n, blocs entiers
  	For m := 0 To (Img.Largeur - 1) Div 16 Do
	  BegiN
      x := m * 16 + 15; y := n * 16 + 15;
    	If x >= Img.Largeur Then x := Img.Largeur - 1;
    	If y >= Img.Hauteur Then y := Img.Hauteur - 1;
	   	If @Progress <> nil Then Progress(Round((n * Img.Largeur / 16 + m) / (Img.Hauteur / 16 * Img.Largeur / 16) * 100));
    	Inc(Adresse, EcrireMBloc(Img, Rect(m * 16, n * 16, x, y), HandleF));
    EnD
  END;
  If TempR <> 0 Then Begin FileWrite(HandleF, TempC, 1); Inc(Adresse) End;
  FileSeek(HandleF, 4, 0);
  FileWrite(HandleF, Adresse, 4);
  FileClose(HandleF);
  Arbre.Destroy;
End;


//*********************************************************************************************
// Dcompression
//*********************************************************************************************


//---------------------------------------------------------------------------------------------
// IDCT - Transforme en cosinus inverse de 'Source' dans 'Resultat'
Procedure IDCT(Var Source, Resultat: Tab8x8);
Const
	UnSurRac2 = 0.7071067812;
Var
	i, j, x, y: Integer;
  Coeff, Fast1, Fast2, Temp: Double;
Begin
	For x := 0 To 7 Do For y := 0 To 7 Do
  Begin
    Temp := 0;
    Fast1 := (x + 0.5) * Pi / 8;
    Fast2 := (y + 0.5) * Pi / 8;
		For i := 0 To 7 Do For j := 0 To 7 Do
    Begin
      Coeff := 1;
      If i = 0 Then Coeff := Coeff * UnSurRac2;
      If j = 0 Then Coeff := Coeff * UnSurRac2;
			Temp := Temp + Coeff * Source[i, j] * cos(Fast1 * i) * cos(Fast2 * j);
    End;
    Resultat[x, y] := Round(Temp * 0.25);
  End
End;


//---------------------------------------------------------------------------------------------
// LireBloc - Ecrit bloc 8x8 pixels suivant du fichier 'HandleF' dans 'Tableau', 'Quant'=Y/Cb/Cr
Function LireBloc(Var Tableau: Tab8x8; NbLC, Quant, HandleF: Integer): Integer;
Var
	i, j: Integer;
  x, y, nbZero, taille, Quant2: Integer;
  TabR: Tab8x8;
  TabLin: Array[0..127] Of Byte;
Begin
	If (NbLC = 8) Or (NbLC = -8) Then
  Begin
    //--- Dlinariser zigzag + RLE (<=> 0) + Huffman adaptative
    Arbre.Actualiser;
    FileRead(HandleF, TabLin, 128);
    nbZero := 0;
		taille := TempR;
    For i := 0 To 63 Do
    Begin
    	x := ZZ[i, 0]; y := ZZ[i, 1];

      If nbZero > 0 Then Begin TabR[x, y] := zero; Dec(nbZero) End Else
      Begin
	      TabR[x, y] := Arbre.LireCar(taille, TabLin);
	      // Dpassement tendue
	      If TabR[x, y] = Gd Then
        Begin
        	TabR[x, y] := LireBits(12, taille, TabLin);
          Dec(TabR[x, y], 2048 - zero);
        End Else
	      // RLE
	      If TabR[x, y] = RLE Then
	      Begin
	      	nbZero := LireBits(6, taille, TabLin);
          TabR[x, y] := zero;
        End
			End;
    End;

    //--- Dquantifier
    If Quant = 0 Then Quant2 := 0 Else Quant2 := 1;
    Inc(TabR[0, 0], DC[Quant]); DC[Quant] := TabR[0, 0] - zero;
  	For i := 0 To 7 Do For j := 0 To 7 Do
      TabR[i, j] := Round((TabR[i, j] - zero) * TQuant[Quant2][i, j]);
   	IDCT(TabR, Tableau);
  End;
  Result := taille Div 8;
	TempR := taille Mod 8;
End;


//---------------------------------------------------------------------------------------------
// LireMBloc - Dessine bloc 16x16 suivant du fichier 'HandleF' sur 'Canevas'  la position 'R'
Procedure LireMBloc(Var Image: sImage; R: TRect; HandleF: Integer; Var Adresse: Integer);
Var
	i, j, k, l, Temp: Integer;
  Ta1, Ta2, Ta3: Tab8x8;
  Pixel: TColor;
  PixLC: sYCbCr;
Begin
  PixLC := sYCbCr.Create;
	//---
  Inc(Adresse, LireBloc(Ta2, 8, 1, HandleF));
  FileSeek(HandleF, Adresse, 0);
  Inc(Adresse, LireBloc(Ta3, 8, 2, HandleF));
  FileSeek(HandleF, Adresse, 0);
	For l := 0 To 1 Do For k := 0 To 1 Do
  Begin
    Inc(Adresse, LireBloc(Ta1, 8, 0, HandleF));
	  FileSeek(HandleF, Adresse, 0);
  	For j := 0 To 7 Do For i := 0 To 7 Do
    Begin
    	Temp := Ta1[i, j];
      If Temp < 0 Then PixLC.Y := 0 Else If Temp > 255 Then PixLC.Y := 255 Else PixLC.Y := Temp;
    	Temp := Ta2[(k * 8 + i) Div 2, (l * 8 + j) Div 2];
      If Temp < 0 Then PixLC.Cb := 0 Else If Temp > 255 Then PixLC.Cb := 255 Else PixLC.Cb := Temp;
    	Temp := Ta3[(k * 8 + i) Div 2, (l * 8 + j) Div 2];
      If Temp < 0 Then PixLC.Cr := 0 Else If Temp > 255 Then PixLC.Cr := 255 Else PixLC.Cr := Temp;
			Pixel := PixLC.ToRGB;
      Image.Pix[R.Left + k * 8 + i, R.Top + l * 8 + j] := Pixel;
    End;
  End;
End;


//---------------------------------------------------------------------------------------------
// ChargerJTPE - Dessine le fichier 'NomFichier' sur 'Canevas'
Procedure ChargerJTPE(NomFichier: String; Var Img: sImage; Progress: TProgressEvent; Fichier: pFichier);
Var
	i, j, m, n, x, y: Integer;
  TailleFichier: Integer;
  HandleF, Adresse: Integer;
  Qualite: Byte;
  Qual: Single;
begin
  Img.nbCouleurs := 16777216;
	Arbre := sHuffman.Create(130);
  DC[0] := 0; DC[1] := 0; DC[2] := 0;
  //--- Lecture en-tte
  HandleF := FileOpen(NomFichier, fmOpenRead);
  Try Try
	  FileRead(HandleF, TailleFichier, 4);
    If TailleFichier <> 1162892362 Then Raise Erreur(Erreur);
	  FileRead(HandleF, TailleFichier, 4);
	  FileRead(HandleF, Img.Largeur, 4);
	  FileRead(HandleF, Img.Hauteur, 4);
	  FileRead(HandleF, Qualite, 1);
 		If Fichier <> nil Then
    Begin
     	Fichier.Nom := ExtractFileName(NomFichier);
		  Fichier.Format := 'Image JTPE (compresse JPEG)';
	    Fichier.Taille := TailleFichier;
	    Fichier.AutreT[0] := 'Qualit (1-100)';
      If Qualite = 0 Then Fichier.AutreC[0] := 'Dfaut' Else Fichier.AutreC[0] := IntToStr(Qualite);
	    Fichier.AutreT[1] := 'Taux comp.';
	    Fichier.AutreC[1] := IntToStr(Round(100 - (TailleFichier - 17) * 100 / (Img.Largeur * Img.Hauteur * 3))) + ' %';
   End;
	  SetLength(Img.Pix, Img.Largeur, Img.Hauteur);
	  Adresse := 17;
	  //--- Cration tables de quantification
	  Qual := 2000 / (Qualite + 16.21) - 17.21;
	  If Qualite = 0 Then For i := 0 To 7 Do For j := 0 To 7 Do
	  	Begin TQuant[0][i, j] := TQuantD[0][i, j]; TQuant[1][i, j] := TQuantD[1][i, j] End Else
	  For i := 0 To 7 Do For j := 0 To 7 Do
	  Begin
	  	TQuant[0][i, j] := 1 + Qual * (1 + i + j);
	  	TQuant[1][i, j] := 1 + Qual * (1 + Sqr(i) + Sqr(j));
	  End;
	  //---
    TempR := 0;
		For n := 0 To (Img.Hauteur - 1) Div 16 Do
	  BEGIN
	  	// Ligne n, blocs entiers
	  	For m := 0 To (Img.Largeur - 1) Div 16 Do
		  BegiN
	      x := m * 16 + 15; y := n * 16 + 15;
	    	If x >= Img.Largeur Then x := Img.Largeur - 1;
	    	If y >= Img.Hauteur Then y := Img.Hauteur - 1;
	     	If @Progress <> nil Then Progress(Round((n * (Img.Largeur / 16) + m) / ((Img.Hauteur / 16) * (Img.Largeur / 16)) * 100));
	    	LireMBloc(Img, Rect(m * 16, n * 16, x, y), HandleF, Adresse);
	    EnD;
	  END;
    Adresse := Adresse;
  Except
  	On Erreur Do ShowMessage('Ce n''est pas le bon type de fichier');
    Else ShowMessage('Il s''est produit une erreur');
  End Finally FileClose(HandleF); Arbre.Destroy End;
End;



end.

